<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Api
{
    const IDEMPOTENCY_KEY_HEADER = 'x-amz-pay-idempotency-key';

    /**
     * @var AmazonPayV2_Client
     */
    protected $_client = null;

    /**
     * @var int
     */
    protected $_storeId = null;

    /**
     * @var int[]
     */
    protected $_validResponses = array(200, 201);

    /**
     * @return Creativestyle_AmazonCheckout_Model_Config
     */
    protected function _getConfig()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Config $config */
        $config = Mage::getSingleton('amazonpay/config');
        return $config;
    }

    /**
     * @return Creativestyle_AmazonCheckout_Model_Url
     */
    protected function _getUrlModel()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Url $urlModel */
        $urlModel = Mage::getSingleton('amazonpay/url');
        return $urlModel;
    }

    /**
     * Return Amazon Pay debug model instance
     *
     * @return Creativestyle_AmazonCheckout_Model_Debug
     */
    protected function _getDebugModel()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Debug $debugModel */
        $debugModel = Mage::getSingleton('amazonpay/debug');
        return $debugModel;
    }

    /**
     * Return instance of Amazon Pay helper
     *
     * @return Creativestyle_AmazonCheckout_Helper_Data
     */
    protected function _getAmazonHelper()
    {
        /** @var Creativestyle_AmazonCheckout_Helper_Data $helper */
        $helper = Mage::helper('amazonpay');
        return $helper;
    }

    /**
     * @return AmazonPayV2_Client
     */
    protected function _getClient()
    {
        if (null === $this->_client) {
            $this->_client = new AmazonPayV2_Client(
                array(
                    'public_key_id' => $this->_getConfig()->getPublicKeyId($this->_storeId),
                    'private_key'   => $this->_getConfig()->getPrivateKey($this->_storeId),
                    'sandbox'       => $this->_getAmazonHelper()->isSandboxActive($this->_storeId),
                    'region'        => $this->_getConfig()->getRegion($this->_storeId)
                )
            );
        }

        return $this->_client;
    }

    /**
     * @param array $headers
     * @return array
     */
    protected function _addIdempotencyKeyHeader(array $headers = array())
    {
        return array_merge(array(self::IDEMPOTENCY_KEY_HEADER => $this->_getAmazonHelper()->generateUuid()), $headers);
    }

    /**
     * @param array $response
     * @return bool
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    protected function  _validateResponse(array $response) {
        if (isset($response['status'])) {
            if (!in_array($response['status'], $this->_validResponses)) {
                $responseData = isset($response['response'])
                    ? json_decode($response['response'], true)
                    : array();

                throw new Creativestyle_AmazonCheckout_Exception_Runtime_Api(
                    isset($responseData['message']) ? $responseData['message'] : 'API error',
                    $response['status'],
                    isset($responseData['reasonCode']) ? $responseData['reasonCode'] : null
                );
            }
        } else {
            throw new Creativestyle_AmazonCheckout_Exception_Runtime_Api('Unrecognized format of API response');
        }
        return true;
    }

    /**
     * @param array $response
     * @return Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
     */
    protected function _parseCheckoutSessionResponse(array $response) {
        return Creativestyle_AmazonCheckout_Model_Api_CheckoutSession::fromJson($response['response']);
    }

    /**
     * @param array $response
     * @return Creativestyle_AmazonCheckout_Model_Api_Buyer
     */
    protected function _parseBuyerResponse(array $response) {
        return Creativestyle_AmazonCheckout_Model_Api_Buyer::fromJson($response['response']);
    }

    protected function _parseJsonResponse(array $response) {
        return json_decode($response['response'], true);
    }

    /**
     * @param int $storeId
     * @return $this
     */
    public function setStoreId($storeId)
    {
        if ($this->_storeId !== $storeId) {
            $this->_storeId = $storeId;
            $this->_client = null;
        }

        return $this;
    }

    /**
     * @param string|null $presentmentCurrency
     * @return Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     * @api
     */
    public function createCheckoutSession($presentmentCurrency = null)
    {
        $response = $this->_getClient()->createCheckoutSession(
            $this->getCheckoutSessionPayload($presentmentCurrency),
            $this->_addIdempotencyKeyHeader()
        );
        $this->_validateResponse($response);
        return $this->_parseCheckoutSessionResponse($response);
    }

    /**
     * @param string $checkoutSessionId
     * @return Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     * @api
     */
    public function getCheckoutSession($checkoutSessionId)
    {
        $response = $this->_getClient()->getCheckoutSession($checkoutSessionId);
        $this->_validateResponse($response);
        return $this->_parseCheckoutSessionResponse($response);
    }

    /**
     * @param string $checkoutSessionId
     * @param float $amount
     * @param string $currency
     * @param null $merchantReferenceId
     * @return Creativestyle_AmazonCheckout_Model_Api_CheckoutSessionInterface
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     * @api
     */
    public function updateCheckoutSession($checkoutSessionId, $amount, $currency, $merchantReferenceId = null)
    {
        $payload = array(
            'webCheckoutDetails' => array(
                'checkoutResultReturnUrl' => $this->_getUrlModel()->getCheckoutResultUrl(),
            ),
            'paymentDetails' => array(
                'paymentIntent' => $this->_getConfig()->shallCaptureImmediately($this->_storeId) ? 'AuthorizeWithCapture' : 'Authorize',
                'canHandlePendingAuthorization' => $this->_getConfig()->isAsyncAuthAllowed($this->_storeId),
                'chargeAmount' => array(
                    'amount' => (float)$amount,
                    'currencyCode' => $currency
                )
            )
        );

        if ($merchantReferenceId) {
            $payload['merchantMetadata'] = array('merchantReferenceId' => $merchantReferenceId);
        }

        $response = $this->_getClient()->updateCheckoutSession($checkoutSessionId, $payload, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseCheckoutSessionResponse($response);
    }

    public function completeCheckoutSession($checkoutSessionId, $amount, $currency)
    {
        $payload = array(
            'chargeAmount' => array(
                'amount' => (float)$amount,
                'currencyCode' => $currency
            )
        );

        $response = $this->_getClient()->completeCheckoutSession($checkoutSessionId, $payload, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseCheckoutSessionResponse($response);
    }

    /**
     * @param string $chargePermissionId
     * @return array
     * @api
     */
    public function getChargePermission($chargePermissionId)
    {
        $response = $this->_getClient()->getChargePermission($chargePermissionId, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseJsonResponse($response);
    }

    /**
     * @param string $chargeId
     * @return array
     * @api
     */
    public function getCharge($chargeId)
    {
        $response = $this->_getClient()->getCharge($chargeId, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseJsonResponse($response);
    }

    public function captureCharge($chargeId, $amount, $currency)
    {
        $payload = array(
            'captureAmount' => array(
                'amount' => (float)$amount,
                'currencyCode' => $currency
            ),
            'softDescriptor' => $this->_getConfig()->getSoftDescriptor($this->_storeId)
        );
        $response = $this->_getClient()->captureCharge($chargeId, $payload, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseJsonResponse($response);
    }

    public function deliveryTrackers($chargePermissionId, $trackingNumber, $carrierCode)
    {
        $payload = array(
            'chargePermissionId' => $chargePermissionId,
            'trackingNumber' => $trackingNumber,
            'carrierCode' => $carrierCode
        );

        $response = $this->_getClient()->deliveryTrackers($payload, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseJsonResponse($response);
    }

    public function createRefund($chargeId, $amount, $currency)
    {
        $payload = array(
            'chargeId' => $chargeId,
            'refundAmount' => array(
                'amount' => (float)$amount,
                'currencyCode' => $currency
            ),
            'softDescriptor' => $this->_getConfig()->getSoftDescriptor($this->_storeId)
        );
        $response = $this->_getClient()->createRefund($payload, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseJsonResponse($response);
    }

    /**
     * @param string|null $presentmentCurrency
     * @return array
     * @api
     */
    public function getCheckoutSessionPayload($presentmentCurrency = null)
    {
        $payload = array(
            'webCheckoutDetails' => array(
                'checkoutReviewReturnUrl' => $this->_getUrlModel()->getCheckoutUrl(),
                'checkoutResultReturnUrl' => $this->_getUrlModel()->getCheckoutResultUrl(),
            ),
            'merchantMetadata' => array(
                'merchantStoreName' => $this->_getConfig()->getStoreName($this->_storeId),
                'customInformation' => sprintf(
                    'Created by creativestyle, %s, V%s',
                    $this->_getDebugModel()->getMagentoVersionString(),
                    $this->_getConfig()->getVersion()
                )
            ),
            'platformId' => 'AIOVPYYF70KB5',
            'storeId' => $this->_getConfig()->getClientId($this->_storeId),
            'deliverySpecifications' => array()
        );

        if (null !== $presentmentCurrency) {
            $payload['paymentDetails'] = array(
                'presentmentCurrency' => $presentmentCurrency
            );
        }

        $allowedCountries = $this->_getConfig()->getAllowedCountries($this->_storeId);

        if (!empty($allowedCountries)) {
            $payload['deliverySpecifications']['addressRestrictions'] = array(
                'type' => 'Allowed',
                'restrictions' => array_combine(
                    $allowedCountries,
                    array_fill(0, count($allowedCountries), new stdClass())
                )
            );
        }

        if ($this->_getConfig()->isPoBoxRestricted($this->_storeId) || $this->_getConfig()->isPackstationRestricted($this->_storeId)) {
            $payload['deliverySpecifications']['specialRestrictions'] = array();
            if ($this->_getConfig()->isPoBoxRestricted($this->_storeId)) {
                $payload['deliverySpecifications']['specialRestrictions'][] = 'RestrictPOBoxes';
            }

            if ($this->_getConfig()->isPackstationRestricted($this->_storeId)) {
                $payload['deliverySpecifications']['specialRestrictions'][] = 'RestrictPackstations';
            }
        }

        return $payload;
    }

    /**
     * @param array|string[] $scopes
     * @return string
     * @api
     */
    public function getSignInPayload(array $scopes = array('name', 'email'))
    {
        return json_encode(array(
            'signInReturnUrl' => $this->_getUrlModel()->getSignInReturnUrl(),
            'storeId' => $this->_getConfig()->getClientId($this->_storeId),
            'signInScopes' => $scopes
        ));
    }

    /**
     * @param string $payload
     * @return string
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     * @api
     */
    public function generateButtonSignature($payload)
    {
        try {
            return $this->_getClient()->generateButtonSignature($payload);
        } catch (Exception $e) {
            throw new Creativestyle_AmazonCheckout_Exception_Runtime_Api($e->getMessage(), 0, null, $e);
        }
    }

    /**
     * @param $buyerToken
     * @return Creativestyle_AmazonCheckout_Model_Api_Buyer
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    public function getBuyer($buyerToken)
    {
        $response = $this->_getClient()->getBuyer($buyerToken, $this->_addIdempotencyKeyHeader());
        $this->_validateResponse($response);
        return $this->_parseBuyerResponse($response);
    }
}
