<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_CheckoutController extends Creativestyle_AmazonCheckout_Controller_Action
{
    /**
     * Return Amazon checkout instance
     *
     * @return Creativestyle_AmazonCheckout_Model_Checkout
     */
    protected function _getCheckout()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Checkout $checkout */
        $checkout = Mage::getSingleton('amazonpay/checkout');
        return $checkout;
    }

    /**
     * @return string
     * @throws Mage_Core_Exception
     */
    protected function _getReviewHtml()
    {
        $layout = $this->getLayout();
        $update = $layout->getUpdate();
        $update->load('amazonpay_checkout_review');
        $layout->generateXml();
        $layout->generateBlocks();
        return $layout->getOutput();
    }

    public function preDispatch()
    {
        $checkoutSessionId = $this->getRequest()
            ->getParam(Creativestyle_AmazonCheckout_Model_Url::PARAM_CHECKOUT_SESSION_ID, null);
        $this->_getCheckout()->setAmazonCheckoutSessionId($checkoutSessionId);
        return parent::preDispatch();
    }

    /**
     * @throws Creativestyle_AmazonCheckout_Exception_InvalidArgument
     * @throws Creativestyle_AmazonCheckout_Exception_Runtime_Api
     */
    public function indexAction()
    {
        if (!$this->_getCheckout()->getQuote()->hasItems() || $this->_getCheckout()->getQuote()->getHasError()) {
            $this->_redirect('checkout/cart');
            return;
        }

        if (null === $this->_getCheckout()->getAmazonCheckoutSessionId()) {
            $this->_redirect('checkout/cart');
            return;
        }

        if (!$this->_getCheckout()->getQuote()->validateMinimumAmount()) {
            $error = Mage::getStoreConfig('sales/minimum_order/error_message') ?
                Mage::getStoreConfig('sales/minimum_order/error_message') :
                Mage::helper('checkout')->__('Subtotal must exceed minimum order amount');
            $this->_getCheckout()->addSessionError($error);
            $this->_redirect('checkout/cart');
            return;
        }

        $checkoutSession = $this->_getCheckout()
            ->importCustomerData()
            ->getAmazonCheckoutSession();

        $this->loadLayout();

        /** @var Creativestyle_AmazonCheckout_Block_Checkout $checkoutBlock */
        $checkoutBlock = $this->getLayout()->getBlock('amazonpay.checkout');
        $checkoutBlock
            ->setPaymentMethodTitle($checkoutSession->getPaymentDescription())
            ->setCheckoutSessionId($checkoutSession->getCheckoutSessionId());

        /** @var Creativestyle_AmazonCheckout_Block_Js $jsBlock */
        $jsBlock = $this->getLayout()->getBlock('amazonpay.js');
        $jsBlock->setCheckoutSessionId($checkoutSession->getCheckoutSessionId());

        $this->renderLayout();
    }

    public function sessionAction()
    {
        if ($this->getRequest()->isPost()) {
            try {
                $checkoutSession = $this->_getCheckout()
                    ->createAmazonCheckoutSession()
                    ->getAmazonCheckoutSession();

                $this->_setJsonResponse(array('checkoutSessionId' => $checkoutSession->getCheckoutSessionId()));
                return;
            } catch (Exception $e) {
                $this->_getLogger()->exception($e);
                $this->_setExceptionResponse($e);
                return;
            }
        }

        $this->_forward('noRoute');
    }

    public function saveShippingMethodAction()
    {
        $shippingMethod = $this->getRequest()->getParam('shipping_method', null);
        $this->_getCheckout()->saveShippingMethod($shippingMethod);
        $this->_setJsonResponse(array('html' => $this->_getReviewHtml()));
    }

    public function couponPostAction()
    {
        if ($this->getRequest()->isPost()) {
            try {
                $couponCode = (string) $this->getRequest()->getParam('coupon_code');

                $this->_getCheckout()->setCouponCode($couponCode);
                $result = array(
                    'success' => 1,
                    'html' => $this->_getReviewHtml()
                );
            } catch (Exception $e) {
                $this->_getLogger()->exception($e);
                $result = array(
                    'success' => 0,
                    'error_messages' => $e->getMessage()
                );
            }
        } else {
            $this->_forward('noRoute');
            return;
        }
        $this->_setJsonResponse($result);
    }

    public function saveAction()
    {
        if ($this->getRequest()->isPost()) {
            try {
                $customFields = $this->getRequest()->getPost('custom_fields', array());
                $customFields = $this->_getAmazonHelper()->filterCheckoutCustomFields($customFields);
                if (!empty($customFields)) {
                    $this->_getCheckout()->setCustomFields($customFields);
                }
                $result = array('success' => 1);
            } catch (Exception $e) {
                $this->_getLogger()->exception($e);
                $result = array(
                    'success' => 0,
                    'error_messages' => $e->getMessage()
                );
            }
        } else {
            $this->_forward('noRoute');
            return;
        }
        $this->_setJsonResponse($result);
    }

    public function placeAction()
    {
        try {
            $checkoutSession = $this->_getCheckout()
                ->updateAmazonCheckoutSession()
                ->getAmazonCheckoutSession();
        } catch (Exception $e) {
            $this->_getLogger()->exception($e);
            $this->_getCheckout()->addSessionError(
                $this->__('There was an error processing your order. Please contact us or try again later.')
            );
            $this->_redirect('checkout/cart');
            return;
        }

        $this->_redirectUrl($checkoutSession->getAmazonPayRedirectUrl());
    }

    public function resultAction()
    {
        try {
            $this->_getCheckout()->completeOrder();
        } catch (Exception $e) {
            $this->_getLogger()->exception($e);
            $this->_getCheckout()->addSessionError(
                $this->__('There was an error processing your order. Please contact us or try again later.')
            );
            $this->_redirect('checkout/cart');
            return;
        }

        $this->_redirect('checkout/onepage/success');
    }
}
